@@ -556,8 +556,8 @@ int rad_accounting_orig(AUTH_REQ *authreq, int authtype, char *f)
 	FILE		*backupfile;
 	struct stat	backup;
 	char		*valbuf;
-	MYSQLREC sqlrecord = {"", "", "", 0, "", "", 0, "", 0, "", "", 0, 0, "", "", "", "", "", "", 0};
-	MYSQLREC backuprecord = {"", "", "", 0, "", "", 0, "", 0, "", "", 0, 0, "", "", "", "", "", "", 0};
+	MYSQLREC sqlrecord = {"", "", "", "", 0, "", "", 0, "", 0, "", "", 0, 0, "", "", "", "", "", "", 0};
+	MYSQLREC backuprecord = {"", "", "",  "", 0, "", "", 0, "", 0, "", "", 0, 0, "", "", "", "", "", "", 0};
 #endif
 	VALUE_PAIR	*pair;
 	int		ret = 0;
@@ -619,6 +619,7 @@ int rad_accounting_orig(AUTH_REQ *authreq, int authtype, char *f)
 #endif
  
 	pair = authreq->request;
+	strncpy(sqlrecord.Realm, authreq->realm, SQLBIGREC);
 	while(pair != (VALUE_PAIR *)NULL) {
 
 				
@@ -764,59 +765,53 @@ int rad_accounting_orig(AUTH_REQ *authreq, int authtype, char *f)
         strncpy(sqlrecord.AcctTimeStamp, datebuf, 20);
        
 
-	if (!mysql_checksocket("Acct")) {
-	     log(L_ERR, "Acct: MySQL connection died");
-	} else {
+	/* If backup file exists we know the database was down */
+	if(stat(MYSQLBACKUP, &backup) == 0) {
+		if(backup.st_size > 0) {
 
- 		/* If backup file exists we know the database was down */
-		if(stat(MYSQLBACKUP, &backup) == 0) {
-			if(backup.st_size > 0) {
+			/* We'll fork a child to load records in the backup file */
+			(pid_t)sqlpid = fork();
+			if(sqlpid > 0) {
 
-                              /* We'll fork a child to load records in the backup file */
+				/* suspend the parent while child reads records */
+				while(waitpid((pid_t)sqlpid, &sqlstatus, 0) != (pid_t)sqlpid);
+			}
+			/* Child Process */
+			if(sqlpid == 0) {
+				if((backupfile = fopen(MYSQLBACKUP, "rwb")) == (FILE *)NULL) {
+					log(L_ERR, "Acct: (Child) Couldn't open file %s", MYSQLBACKUP);
+					exit(1);
+				}
 
-                               (pid_t)sqlpid = fork();
-                               if(sqlpid > 0) {
+				/* Lock the mysql backup file, prefer lockf() over flock(). */
+#if defined(F_LOCK) && !defined( BSD)
+				(void)lockf((int)backupfile, (int)F_LOCK, (off_t)SQL_LOCK_LEN);
+#else
+				(void)flock(backupfile, SQL_LOCK_EX);
+#endif  
 
-				  /* suspend the parent while child reads records */
-                                  while(waitpid((pid_t)sqlpid, &sqlstatus, 0) != (pid_t)sqlpid);
-                               }
-                                /* Child Process */
-                                if(sqlpid == 0) {
-                                   if((backupfile = fopen(MYSQLBACKUP, "rwb")) == (FILE *)NULL) {
-                                      log(L_ERR, "Acct: (Child) Couldn't open file %s", MYSQLBACKUP);
-                                      exit(1);
-                                   }
+				log(L_INFO, "Acct:  Clearing out sql backup file - %s", MYSQLBACKUP);
 
-                                   /* Lock the mysql backup file, prefer lockf() over flock(). */
-                                   #if defined(F_LOCK) && !defined( BSD)
-                                     (void)lockf((int)backupfile, (int)F_LOCK, (off_t)SQL_LOCK_LEN);
-                                   #else
-                                     (void)flock(backupfile, SQL_LOCK_EX);
-                                   #endif  
+				while(!feof(backupfile)) {
+					if(fread(&backuprecord, sizeof(MYSQLREC), 1, backupfile) == 1) {
 
-                                   log(L_INFO, "Acct:  Clearing out sql backup file - %s", MYSQLBACKUP);
+						/* pass our filled structure to the
+					 	   function that will write to the database */
+						ret = mysql_save_acct(&backuprecord);
 
-                                   while(!feof(backupfile)) {
-                                       if(fread(&backuprecord, sizeof(MYSQLREC), 1, backupfile) == 1) {
+					}
 
-                                            /* pass our filled structure to the function that will write to the database */
-                                            ret = mysql_save_acct(backuprecord);
-
-                                        }
-
-                                   }
-                                   unlink((const char *)MYSQLBACKUP);
-                                   exit(0);
-                                }
-                            }
+				}
+				unlink((const char *)MYSQLBACKUP);
+				exit(0);
+			}
 		}
-
-		ret = mysql_save_acct(sqlrecord);
-		if (!mysql_keepopen) {
-			mysql_close(MyAcctSock);
-			MyAcctSock = NULL;
-		}
-        } 
+	}
+	ret = mysql_save_acct(&sqlrecord);
+	if (!mysql_keepopen) {
+		mysql_close(MyAcctSock);
+		MyAcctSock = NULL;
+	}
 #endif
 
 	return ret;
