@@ -548,18 +548,22 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 	int		result, r;
 	char		pw_digest[16];
 	char		userpass[128];
-#ifdef USEMYSQL
-	char		dbpass[128];
-#endif
 	char		umsg[128];
 	char		*user_msg;
 	char		*ptr;
 	char		*exec_program;
 	int		exec_wait;
 	int		seen_callback_id;
+#ifdef USEMYSQL
+	char		string[AUTH_STRING_LEN];
+	char		dbpass[128];
+	int 		i;
+#endif
 
 	user_check = NULL;
 	user_reply = NULL;
+	user_msg = NULL;
+	userpass[0] = 0;
 
 	/*
 	 *	Get the username from the request.
@@ -606,59 +610,239 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 		user_reply = authreq->server_reply;
 
 
-	/*
-	 *	Get the user from the database
-	 */
-        if (MyAuthSock != NULL )
- 	 	DEBUG("MyAuthSock is NOT null before user_find");
-	if (user_find(namepair->strvalue, authreq->request,
-	   &user_check, &user_reply) != 0) {
-		log(L_AUTH, "Invalid user: [%s] (%s)",
-			namepair->strvalue, auth_name(authreq, 1));
-		rad_send_reply(PW_AUTHENTICATION_REJECT, authreq,
-			proxy_pairs, NULL, activefd);
-		pairfree(proxy_pairs);
-		pairfree(user_check);
-		pairfree(user_reply);
-		authfree(authreq);
-		return -1;
-	}
-
-	/*
-	 *	Validate the user
-	 */
-	user_msg = NULL;
-	userpass[0] = 0;
-	do {
-		if ((result = check_expiration(user_check, umsg, &user_msg))<0)
-				break;
-
-		result = rad_check_password(authreq, activefd, user_check,
-			namepair, pw_digest, &user_msg, userpass
 #ifdef USEMYSQL
-			, dbpass);
-#else
-			);
-#endif
+	if (authreq->server_code) {
 
-		if (result > 0) {
-			/*
-			 *	FIXME: proxy_pairs is lost in reply.
-			 *	Happens only in the CHAP case, which
-			 *	doesn't work anyway, so ..
-			 */
-			authfree(authreq);
+		VALUE_PAIR	*realm_check = NULL;
+
+		auth_item = pairfind(authreq->request, PW_PASSWORD);
+		/*
+		 *	Decrypt the password.
+		 */
+		if (auth_item != NULL) {
+			memcpy(string, auth_item->strvalue, AUTH_PASS_LEN);
+			for(i = 0;i < AUTH_PASS_LEN;i++)
+				string[i] ^= pw_digest[i];
+			string[AUTH_PASS_LEN] = '\0';
+			strcpy(userpass, string);
+		}
+		mysql_getvpdata(mysql_groupcheck_table, &realm_check, authreq->realm, PW_VP_REALMDATA);
+		mysql_getvpdata(mysql_groupreply_table, &user_reply, authreq->realm, PW_VP_REALMDATA);
+
+		if (authreq->server_code == PW_AUTHENTICATION_ACK) {
+			result = 0;
+
+			if((result = check_expiration(realm_check, umsg, &user_msg)) < 0) {
+					rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, user_reply, user_msg, activefd); 
+					log(L_AUTH, "Realm Expired: [%s] (%s)", authreq->realm, auth_name(authreq, 1));
+					return -1;
+			}
+
+			if (result >= 0 && (check_item = pairfind(realm_check, PW_SIMULTANEOUS_USE)) != NULL) {
+
+				MYSQL_ROW	row;
+				MYSQL_RES	*sqlresult;
+				char		querystr[256];
+				int		num = 0;
+
+				mysql_checksocket("Auth");
+				sprintf(querystr, "SELECT COUNT(*) FROM radacct WHERE Realm = '%s' and AcctStopTime = 0", authreq->realm);
+				if (sqltrace)
+					DEBUG(querystr);
+				mysql_query(MyAuthSock, querystr);
+				if (!(sqlresult = mysql_store_result(MyAuthSock)) && mysql_num_fields(MyAuthSock)) {
+					log(L_ERR,"MYSQL Error: Cannot get result");
+					log(L_ERR,"MYSQL error: %s",mysql_error(MyAuthSock));
+					mysql_close(MyAuthSock);
+					MyAuthSock = NULL;
+				} else {
+					row = mysql_fetch_row(sqlresult);
+					num = atol(row[0]);
+					mysql_free_result(sqlresult);
+				}
+				if (num >= check_item->lvalue) {
+					sprintf(umsg, "\r\nYour realm has reached it's limit of %d simultaneous logins\r\n\n", (int)check_item->lvalue);
+					user_msg = umsg;
+					result = -1;
+					rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, user_reply, user_msg, activefd);
+					log(L_ERR, "Realm multiple logins: [%s] (%s) max. %d", authreq->realm, auth_name(authreq, 1), check_item->lvalue);
+				}
+			}
+
+			if (result >=0 && (check_item = pairfind(realm_check, PW_MAX_HOURS)) != NULL) {
+
+				MYSQL_ROW	row;
+				MYSQL_RES	*sqlresult;
+				char		querystr[256];
+				unsigned long	allowed;
+				unsigned long	diff;
+				unsigned long	used = 0;
+	
+				allowed = check_item->lvalue * 3600;
+				mysql_checksocket("Auth");
+				sprintf(querystr, "SELECT SUM(AcctSessionTime) FROM radacct WHERE Realm = '%s' and AcctStartTime >= CONCAT(YEAR(NOW()),'-',MONTH(NOW()),'-01 00:00:00') AND AcctStartTime <= CONCAT(YEAR(NOW()),'-',MONTH(NOW())+1,'-01 00:00:00')", authreq->realm);
+				if (sqltrace)
+					DEBUG(querystr);
+				mysql_query(MyAuthSock, querystr);
+				if (!(sqlresult = mysql_store_result(MyAuthSock)) && mysql_num_fields(MyAuthSock)) {
+					log(L_ERR,"MYSQL Error: Cannot get result");
+					log(L_ERR,"MYSQL error: %s",mysql_error(MyAuthSock));
+					mysql_close(MyAuthSock);
+					MyAuthSock = NULL;
+				} else {
+					row = mysql_fetch_row(sqlresult);
+					used = atol(row[0]);
+					mysql_free_result(sqlresult);
+				}
+				
+				if (used >= allowed) {
+					sprintf(umsg, "\r\nYour realm has reached it's limit of %d hours\r\n\n", (int)check_item->lvalue);
+					user_msg = umsg;
+					result = -1;
+					rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, user_reply, user_msg, activefd);
+					log(L_ERR, "Realm usage limit of %d hours reached: [%s] (%s)", (int)check_item->lvalue, authreq->realm, auth_name(authreq, 1));
+				}
+			
+				if (result >=0 && (reply_item = pairfind(user_reply, PW_SESSION_TIMEOUT)) != NULL) {
+					diff = allowed - used;
+					if (diff < reply_item->lvalue)
+						reply_item->lvalue = diff;
+				} else if (result >= 0) {
+					if (!(reply_item = malloc(sizeof(VALUE_PAIR)))){
+						log(L_ERR|L_CONS, "no memory");
+						exit(1);
+					}
+					memset(reply_item, 0, sizeof(VALUE_PAIR));
+					reply_item->attribute = PW_SESSION_TIMEOUT;
+					strcpy(reply_item->name, "Session-Timeout");
+					reply_item->type = PW_TYPE_INTEGER;
+					reply_item->length = 4;
+					reply_item->lvalue = diff;
+					pairadd(&user_reply, reply_item);
+				}
+
+			}
+
+			if (result >= 0 &&
+			   (check_item = pairfind(realm_check, PW_LOGIN_TIME)) != NULL) {
+	
+				/*
+				 *	Authentication is OK. Now see if this
+				 *	realm may login at this time of the day.
+				 */
+				r = timestr_match(check_item->strvalue, time(NULL));
+				if (r < 0) {
+					/*
+					 *	User called outside allowed time interval.
+					 */
+					result = -1;
+					user_msg =
+					"You are calling outside your allowed timespan\r\n";
+					rad_send_reply(PW_AUTHENTICATION_REJECT, authreq,
+						user_reply, user_msg, activefd);
+					log(L_ERR, "Realm Outside allowed timespan: [%s]"
+						   " (from nas %s) time allowed: %s",
+							authreq->realm,
+							nas_name2(authreq),
+							check_item->strvalue);
+				} else if (r > 0) {
+					/*
+					 *	User is allowed, but set Session-Timeout.
+					 */
+					if ((reply_item = pairfind(user_reply,
+					    PW_SESSION_TIMEOUT)) != NULL) {
+						if (reply_item->lvalue > r)
+							reply_item->lvalue = r;
+					} else {
+						if (!(reply_item = malloc(sizeof(VALUE_PAIR)))){
+							log(L_ERR|L_CONS, "no memory");
+							exit(1);
+						}
+						memset(reply_item, 0, sizeof(VALUE_PAIR));
+						reply_item->attribute = PW_SESSION_TIMEOUT;
+						strcpy(reply_item->name, "Session-Timeout");
+						reply_item->type = PW_TYPE_INTEGER;
+						reply_item->length = 4;
+						reply_item->lvalue = r;
+						pairadd(&user_reply, reply_item);
+					}
+				}
+			}
+			if (result < 0) {
+				authfree(authreq);
+				pairfree(realm_check);
+				pairfree(user_reply);
+				return 0;
+			}
+
+		} else if (authreq->server_code == PW_AUTHENTICATION_REJECT) {
+			pairfree(realm_check);
+			strncpy(dbpass, "UNKNOWN", 7);
+			result = -1;
+		}
+	} else {
+#endif
+		/*
+		 *	Get the user from the database
+		 */
+		if (user_find(namepair->strvalue, authreq->request,
+		   &user_check, &user_reply) != 0) {
+			log(L_AUTH, "Invalid user: [%s] (%s)",
+				namepair->strvalue, auth_name(authreq, 1));
+			rad_send_reply(PW_AUTHENTICATION_REJECT, authreq,
+				proxy_pairs, NULL, activefd);
 			pairfree(proxy_pairs);
 			pairfree(user_check);
 			pairfree(user_reply);
+			authfree(authreq);
 			return -1;
 		}
-		if (result == -2) {
-			if ((reply_item = pairfind(user_reply,
-			     PW_REPLY_MESSAGE)) != NULL)
-				user_msg = reply_item->strvalue;
-		}
-	} while(0);
+
+		/*
+		 *	Validate the user
+		 */
+		do {
+			if ((result = check_expiration(user_check, umsg, &user_msg))<0)
+#ifdef USEMYSQL
+			{
+					rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, user_reply, user_msg, activefd); 
+					log(L_AUTH, "Account Expired: [%s] (%s)", namepair->strvalue, auth_name(authreq, 1));
+					return -1;
+			}
+#else
+					break;
+#endif
+
+			result = rad_check_password(authreq, activefd, user_check,
+				namepair, pw_digest, &user_msg, userpass
+#ifdef USEMYSQL
+				, dbpass);
+#else
+				);
+#endif
+
+			if (result > 0) {
+				/*
+				 *	FIXME: proxy_pairs is lost in reply.
+				 *	Happens only in the CHAP case, which
+				 *	doesn't work anyway, so ..
+				 */
+				authfree(authreq);
+				pairfree(proxy_pairs);
+				pairfree(user_check);
+				pairfree(user_reply);
+				return -1;
+			}
+			if (result == -2) {
+				if ((reply_item = pairfind(user_reply,
+				     PW_REPLY_MESSAGE)) != NULL)
+					user_msg = reply_item->strvalue;
+			}
+		} while(0);
+
+#ifdef USEMYSQL
+	}
+#endif
 
 	pairmove2(&user_reply, &proxy_pairs, PW_PROXY_STATE);
 
@@ -869,6 +1053,8 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 	 *	FIXME: somehow cache this info, and only execute the
 	 *	program when we receive an Accounting-START packet.
 	 *	Only at that time we know dynamic IP etc.
+	 *
+	 *	FIXME: ICRADIUS support realm to Exec-Program (as %R ?)
 	 */
 	exec_program = NULL;
 	exec_wait = 0;
