@@ -1,5 +1,5 @@
 /*
- * mysql.c	MySQL routines.
+ * mysql.c	MySQL routines. Used bt ICRADIUS 0.7 extensions
  *
  */
 #ifdef USEMYSQL
@@ -56,8 +56,10 @@ int mysql_start ()
        strcpy(mysql_groupcheck_table,"radgroupcheck");
        strcpy(mysql_groupreply_table,"radgroupreply");
        strcpy(mysql_usergroup_table,"usergroup");
+       strcpy(mysql_realmgroup_table,"realmgroup");
        strcpy(mysql_acct_table,"radacct");
        strcpy(mysql_nas_table,"nas");
+       strcpy(mysql_realm_table, "realms");
        strcpy(mysql_dict_table,"dictionary");
        sqltrace = 0;
        mysql_keepopen = 0;
@@ -110,7 +112,7 @@ int mysql_start ()
                        }
                }
                 if(strncasecmp(buffer, "authcheck_table", 15) == 0) {
-                        /* Read the ACCT_TABLE line */
+                        /* Read the AUTHCHECK_TABLE line */
                         if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
                                log(L_ERR,"invalid attribute on line %d of sqlserver file %s", line_no,sqlfile);
                        } else {
@@ -118,7 +120,7 @@ int mysql_start ()
                        }
                }
                 if(strncasecmp(buffer, "authreply_table", 15) == 0) {
-                        /* Read the ACCT_TABLE line */
+                        /* Read the AUTHREPLY_TABLE line */
                         if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
                                log(L_ERR,"invalid attribute on line %d of sqlserver file %s", line_no,sqlfile);
                        } else {
@@ -126,7 +128,7 @@ int mysql_start ()
                        }
                }
                 if(strncasecmp(buffer, "groupcheck_table", 16) == 0) {
-                        /* Read the ACCT_TABLE line */
+                        /* Read the GROUPCHECK_TABLE line */
                         if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
                                log(L_ERR,"invalid attribute on line %d of sqlserver file %s", line_no,sqlfile);
                        } else {
@@ -134,7 +136,7 @@ int mysql_start ()
                        }
                }
                 if(strncasecmp(buffer, "groupreply_table", 16) == 0) {
-                        /* Read the ACCT_TABLE line */
+                        /* Read the GROUP_REPLY_TABLE line */
                         if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
                                log(L_ERR,"invalid attribute on line %d of sqlserver file %s", line_no,sqlfile);
                        } else {
@@ -142,12 +144,20 @@ int mysql_start ()
                        }
                }
                 if(strncasecmp(buffer, "usergroup_table", 15) == 0) {
-                        /* Read the ACCT_TABLE line */
+                        /* Read the USERGROUP_TABLE line */
                         if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
                                log(L_ERR,"invalid attribute on line %d of sqlserver file %s", line_no,sqlfile);
                        } else {
                          strncpy(mysql_usergroup_table,namestr, MAX_TABLE_LEN);
                        }
+               }
+                if(strncasecmp(buffer, "realmgroup_table", 16) == 0) {
+                        /* Read the REALMGROUP_TABLE line */
+                        if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
+                               log(L_ERR,"invalid attribute on line %d of sqlserver file %s", line_no,sqlfile);
+                       } else {
+                         strncpy(mysql_realmgroup_table,namestr, MAX_TABLE_LEN);
+                       }
                }
                 if(strncasecmp(buffer, "acct_table", 10) == 0) {
                         /* Read the ACCT_TABLE line */
@@ -164,6 +174,14 @@ int mysql_start ()
                        } else {
                          strncpy(mysql_nas_table,namestr, MAX_TABLE_LEN);
                        }
+               }
+                if(strncasecmp(buffer, "realm_table", 9) == 0) {
+                       /* Read the REALM_TABLE line */
+                       if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
+                               log(L_ERR,"invalid attribute on line %d of sqlserver file %s", line_no,sqlfile);
+                      } else {
+                         strncpy(mysql_realm_table,namestr, MAX_TABLE_LEN);
+                      }
                }
                 if(strncasecmp(buffer, "dict_table", 9) == 0) {
                         /* Read the DICT_TABLE line */
@@ -232,13 +250,17 @@ int mysql_start ()
  *
  *************************************************************************/
 
-int mysql_save_acct(MYSQLREC sqlrecord) {
+int mysql_save_acct(MYSQLREC *sqlrecord) {
 
 	char		querystr[2048];
 	FILE		*mysqlfile;
 	FILE		*backupfile;
 	int		num = 0;
 	MYSQL_RES	*result;
+#ifdef NT_DOMAIN_HACK
+	char		*ptr;
+	char		newname[AUTH_STRING_LEN];
+#endif
 	
 
 
@@ -252,61 +274,69 @@ int mysql_save_acct(MYSQLREC sqlrecord) {
         #endif
      }
 
-     if (sqlrecord.AcctStatusTypeId == PW_STATUS_ACCOUNTING_ON || sqlrecord.AcctStatusTypeId == PW_STATUS_ACCOUNTING_OFF) {
-        log(L_INFO, "Portmaster %s rebooted at %s", sqlrecord.NASIPAddress, sqlrecord.AcctTimeStamp);
-    
-   	if (MyAcctSock != NULL) {
+#ifdef NT_DOMAIN_HACK
+	/*
+	 *      Windows NT machines often authenticate themselves as
+	 *      NT_DOMAIN\username. Try to be smart about this.
+	 *
+	 *      FIXME: should we handle this as a REALM ?
+	 */
+	if ((ptr = strchr(sqlrecord->UserName, '\\')) != NULL) {
+		strncpy(newname, ptr + 1, sizeof(newname));
+		newname[sizeof(newname) - 1] = 0;
+		strcpy(sqlrecord->UserName, newname);
+	}
+#endif /* NT_DOMAIN_HACK */
+
+ if (mysql_checksocket("Acct")) {
+
+     if (sqlrecord->AcctStatusTypeId == PW_STATUS_ACCOUNTING_ON || sqlrecord->AcctStatusTypeId == PW_STATUS_ACCOUNTING_OFF) {
+        log(L_INFO, "Portmaster %s rebooted at %s", sqlrecord->NASIPAddress, sqlrecord->AcctTimeStamp);
   
-          /* The Terminal server informed us that it was rebooted
-          * STOP all records from this NAS */
+         /* The Terminal server informed us that it was rebooted
+         * STOP all records from this NAS */
 
-             sprintf(querystr, "UPDATE %s SET AcctStopTime='%s', AcctSessionTime=unix_timestamp('%s') - unix_timestamp(AcctStartTime), AcctTerminateCause='%s' WHERE AcctSessionTime=0 AND AcctStopTime=0 AND NASIPAddress= '%s' AND AcctStartTime <= '%s'", mysql_acct_table, sqlrecord.AcctTimeStamp, sqlrecord.AcctTimeStamp, sqlrecord.AcctTerminateCause, sqlrecord.NASIPAddress, sqlrecord.AcctTimeStamp);
+         sprintf(querystr, "UPDATE %s SET AcctStopTime='%s', AcctSessionTime=unix_timestamp('%s') - unix_timestamp(AcctStartTime), AcctTerminateCause='%s' WHERE AcctSessionTime=0 AND AcctStopTime=0 AND NASIPAddress= '%s' AND AcctStartTime <= '%s'", mysql_acct_table, sqlrecord->AcctTimeStamp, sqlrecord->AcctTimeStamp, sqlrecord->AcctTerminateCause, sqlrecord->NASIPAddress, sqlrecord->AcctTimeStamp);
 
-       	      if (mysql_query(MyAcctSock, (const char *) querystr) < 0)
-	          log(L_ERR, "Acct: Couldn't update SQL accounting after NAS reboot - %s", mysql_error(MyAcctSock));
+ 	 if (sqltrace)
+	      DEBUG(querystr);
+       	 if (mysql_query(MyAcctSock, (const char *) querystr) < 0)
+	      log(L_ERR, "Acct: Couldn't update SQL accounting after NAS reboot - %s", mysql_error(MyAcctSock));
 
-              if (mysqlfile) {
-                  fputs(querystr, mysqlfile);
-                  fputs(";\n", mysqlfile);
-                  fclose(mysqlfile);
-              }
+         if (mysqlfile) {
+              fputs(querystr, mysqlfile);
+              fputs(";\n", mysqlfile);
+              fclose(mysqlfile);
+          }
           return 0;
-        }
+      } 
 
+	if (sqlrecord->AcctStatusTypeId == PW_STATUS_ALIVE) {
+		sprintf(querystr, "UPDATE %s SET Framed-IP-Address = '%s' WHERE AcctSessionId = '%s' AND UserName = '%s' AND NASIPAddress= '%s'", mysql_acct_table, sqlrecord->FramedIPAddress, sqlrecord->AcctSessionId, sqlrecord->UserName, sqlrecord->NASIPAddress);
+		if (sqltrace)
+			DEBUG(querystr);
+		if (mysql_query(MyAcctSock, (const char *) querystr) < 0)
+		log(L_ERR, "Acct: Couldn't update SQL accounting after NAS reboot - %s", mysql_error(MyAcctSock));
 
-        /* we know database is down becuase the earlier return would have executed */
-        if((backupfile = fopen(MYSQLBACKUP, "a")) == (FILE *)NULL) {
-             log(L_ERR, "Acct: Couldn't open file %s", MYSQLBACKUP);
-        } else {
-             /*
-              * Lock the mysql backup file, prefer lockf() over flock().
-              */
-              #if defined(F_LOCK) && !defined(BSD)
-                  (void)lockf((int)backupfile, (int)F_LOCK, (off_t)SQL_LOCK_LEN);
-              #else
-                  (void)flock(backupfilefile, SQL_LOCK_EX);
-              #endif
-              if(fwrite(&sqlrecord, sizeof(MYSQLREC), 1, backupfile) < 1) {
-                  log(L_ERR, "Acct: Couldn't write to file %s", MYSQLBACKUP);
-              }
-              fclose(backupfile);
-         }
+		if (mysqlfile) {
+			fputs(querystr, mysqlfile);
+			fputs(";\n", mysqlfile);
+			fclose(mysqlfile);
+		}
+		return 0;
+	}
 
-        return 0;
-     } 
-
-     if (MyAcctSock != NULL) {
 
           /* Got start record */
-          if(sqlrecord.AcctStatusTypeId == PW_STATUS_START) {
+          if(sqlrecord->AcctStatusTypeId == PW_STATUS_START) {
              
              /* Set start time on record with only a stop record */
  	     snprintf(querystr, 2048, "UPDATE %s SET AcctStartTime = '%s' WHERE AcctSessionId = '%s' AND UserName = '%s' AND NASIPAddress = '%s'", 
 	     mysql_acct_table,
-             sqlrecord.AcctTimeStamp,
-             sqlrecord.AcctSessionId,
-             sqlrecord.UserName,
-             sqlrecord.NASIPAddress
+             sqlrecord->AcctTimeStamp,
+             sqlrecord->AcctSessionId,
+             sqlrecord->UserName,
+             sqlrecord->NASIPAddress
              );
  	     if (sqltrace)
 	        DEBUG(querystr);
@@ -317,22 +347,23 @@ int mysql_save_acct(MYSQLREC sqlrecord) {
              if (num == 0) {
 
                 /* Insert new record with blank stop time until stop record is got */
-                snprintf(querystr, 2048, "INSERT INTO %s VALUES (0, '%s', '%s', '%s', %ld, '%s', '%s', 0, 0, '%s', '%s', 0, 0, '%s', '%s', '', '%s', '%s', '%s', %ld)",
+                snprintf(querystr, 2048, "INSERT INTO %s VALUES (0, '%s', '%s', '%s', '%s', %ld, '%s', '%s', 0, 0, '%s', '%s', 0, 0, '%s', '%s', '', '%s', '%s', '%s', %ld)",
                 mysql_acct_table,
-                sqlrecord.AcctSessionId,
-                sqlrecord.UserName,
-                sqlrecord.NASIPAddress,
-                sqlrecord.NASPortId,
-                sqlrecord.NASPortType,
-                sqlrecord.AcctTimeStamp,
-                sqlrecord.AcctAuthentic,
-                sqlrecord.ConnectInfo,
-                sqlrecord.CalledStationId,
-                sqlrecord.CallingStationId,
-                sqlrecord.ServiceType,
-                sqlrecord.FramedProtocol,
-                sqlrecord.FramedIPAddress,
-                sqlrecord.AcctDelayTime
+                sqlrecord->AcctSessionId,
+                sqlrecord->UserName,
+                sqlrecord->Realm,
+                sqlrecord->NASIPAddress,
+                sqlrecord->NASPortId,
+                sqlrecord->NASPortType,
+                sqlrecord->AcctTimeStamp,
+                sqlrecord->AcctAuthentic,
+                sqlrecord->ConnectInfo,
+                sqlrecord->CalledStationId,
+                sqlrecord->CallingStationId,
+                sqlrecord->ServiceType,
+                sqlrecord->FramedProtocol,
+                sqlrecord->FramedIPAddress,
+                sqlrecord->AcctDelayTime
                 );                  
 
  		if (sqltrace)
@@ -344,7 +375,7 @@ int mysql_save_acct(MYSQLREC sqlrecord) {
            /* Got stop record */
            } else {
 
-             sprintf(querystr, "SELECT RadAcctId FROM %s WHERE AcctSessionId='%s' AND NASIPAddress='%s' AND UserName='%s'", mysql_acct_table, sqlrecord.AcctSessionId, sqlrecord.NASIPAddress, sqlrecord.UserName);
+             sprintf(querystr, "SELECT RadAcctId FROM %s WHERE AcctSessionId='%s' AND NASIPAddress='%s' AND UserName='%s'", mysql_acct_table, sqlrecord->AcctSessionId, sqlrecord->NASIPAddress, sqlrecord->UserName);
  	      if (sqltrace)
   	        DEBUG(querystr);
               mysql_query(MyAcctSock, querystr);
@@ -363,14 +394,14 @@ int mysql_save_acct(MYSQLREC sqlrecord) {
                 /* Set stop time on matching record with start time */
  	        snprintf(querystr, 2048, "UPDATE %s SET AcctStopTime = '%s', AcctSessionTime = '%lu', AcctInputOctets = '%u', AcctOutputOctets = '%u', AcctTerminateCause = '%s' WHERE AcctSessionId = '%s' AND UserName = '%s' AND NASIPAddress = '%s'", 
 	        mysql_acct_table,
-                sqlrecord.AcctTimeStamp,
-                sqlrecord.AcctSessionTime,
-                sqlrecord.AcctInputOctets,
-                sqlrecord.AcctOutputOctets,
-                sqlrecord.AcctTerminateCause,
-                sqlrecord.AcctSessionId,
-                sqlrecord.UserName,
-                sqlrecord.NASIPAddress
+                sqlrecord->AcctTimeStamp,
+                sqlrecord->AcctSessionTime,
+                sqlrecord->AcctInputOctets,
+                sqlrecord->AcctOutputOctets,
+                sqlrecord->AcctTerminateCause,
+                sqlrecord->AcctSessionId,
+                sqlrecord->UserName,
+                sqlrecord->NASIPAddress
                 );
 
 
@@ -383,26 +414,27 @@ int mysql_save_acct(MYSQLREC sqlrecord) {
 
             
                 /* Insert record with no start time until matching start record comes */
-                snprintf(querystr, 2048, "INSERT INTO %s VALUES (0, '%s', '%s', '%s', %ld, '%s', 0, '%s', '%lu', '%s', '%s', '%u', '%u', '%s', '%s', '%s', '%s', '%s', '%s', %ld)",
+                snprintf(querystr, 2048, "INSERT INTO %s VALUES (0, '%s', '%s', '%s', '%s', %ld, '%s', 0, '%s', '%lu', '%s', '%s', '%u', '%u', '%s', '%s', '%s', '%s', '%s', '%s', %ld)",
                 mysql_acct_table,
-                sqlrecord.AcctSessionId,
-                sqlrecord.UserName,
-                sqlrecord.NASIPAddress,
-                sqlrecord.NASPortId,
-                sqlrecord.NASPortType,
-	        sqlrecord.AcctTimeStamp,
-		sqlrecord.AcctSessionTime,
-                sqlrecord.AcctAuthentic,
-                sqlrecord.ConnectInfo,
-		sqlrecord.AcctInputOctets,
-		sqlrecord.AcctOutputOctets,
-                sqlrecord.CalledStationId,
-                sqlrecord.CallingStationId,
-		sqlrecord.AcctTerminateCause,
-                sqlrecord.ServiceType,
-                sqlrecord.FramedProtocol,
-                sqlrecord.FramedIPAddress,
-                sqlrecord.AcctDelayTime
+                sqlrecord->AcctSessionId,
+                sqlrecord->UserName,
+                sqlrecord->Realm,
+                sqlrecord->NASIPAddress,
+                sqlrecord->NASPortId,
+                sqlrecord->NASPortType,
+	        sqlrecord->AcctTimeStamp,
+		sqlrecord->AcctSessionTime,
+                sqlrecord->AcctAuthentic,
+                sqlrecord->ConnectInfo,
+		sqlrecord->AcctInputOctets,
+		sqlrecord->AcctOutputOctets,
+                sqlrecord->CalledStationId,
+                sqlrecord->CallingStationId,
+		sqlrecord->AcctTerminateCause,
+                sqlrecord->ServiceType,
+                sqlrecord->FramedProtocol,
+                sqlrecord->FramedIPAddress,
+                sqlrecord->AcctDelayTime
                 );                  
 
  	        if (sqltrace)
@@ -438,7 +470,7 @@ int mysql_save_acct(MYSQLREC sqlrecord) {
                    #else
                        (void)flock(backupfile, SQL_LOCK_EX);
                    #endif
-                   if(fwrite(&sqlrecord, sizeof(MYSQLREC), 1, backupfile) < 1) {
+                   if(fwrite(sqlrecord, sizeof(MYSQLREC), 1, backupfile) < 1) {
                        log(L_ERR, "Acct: Couldn't write to file %s", MYSQLBACKUP);
                    }
                    fclose(backupfile);
@@ -458,7 +490,7 @@ int mysql_save_acct(MYSQLREC sqlrecord) {
  *	Purpose: Read entries from the database and fill VALUE_PAIR structures
  *
  *************************************************************************/
-int mysql_userparse(VALUE_PAIR **first_pair, MYSQL_ROW row, int mode) {
+int mysql_userparse(VALUE_PAIR **first_pair, MYSQL_ROW row) {
 
 	int x;
 	char		*s;
@@ -648,6 +680,8 @@ int mysql_getvpdata(char *table, VALUE_PAIR **vp, char *user, int mode) {
 		sprintf(querystr, "SELECT * FROM %s WHERE UserName = '%s'", table, user);
 	else if (mode == PW_VP_GROUPDATA)
 		sprintf(querystr, "SELECT %s.* FROM %s, %s WHERE %s.UserName = '%s' AND %s.GroupName = %s.GroupName ORDER BY %s.id", table, table, mysql_usergroup_table, mysql_usergroup_table, user, mysql_usergroup_table, table, table);
+	else if (mode == PW_VP_REALMDATA)
+		sprintf(querystr, "SELECT %s.* FROM %s, %s WHERE %s.RealmName = '%s' AND %s.GroupName = %s.GroupName ORDER BY %s.id", table, table, mysql_realmgroup_table, mysql_realmgroup_table, user, mysql_realmgroup_table, table, table);
 	if (sqltrace)
 		DEBUG(querystr);
         mysql_checksocket("Auth");
@@ -661,7 +695,7 @@ int mysql_getvpdata(char *table, VALUE_PAIR **vp, char *user, int mode) {
 		rows = mysql_num_rows(result);
 		while ((row = mysql_fetch_row(result))) {
 
-			if (mysql_userparse(vp, row, PW_VP_REPLY) != 0) {
+			if (mysql_userparse(vp, row) != 0) {
 		 		log(L_ERR|L_CONS, "Error getting data from MySQL");
 				mysql_free_result(result);
 				return -1;
@@ -701,7 +735,7 @@ static int mysql_check_ts(MYSQL_ROW row) {
 	 *      Find NAS type.
 	 */
 	if ((nas = nas_find(ipstr2long(row[3]))) == NULL) {
-                log(L_ERR, "Accounting: unknown NAS");
+                log(L_ERR, "Accounting: unknown NAS [%s]", row[3]);
                 return -1;
         }
 
@@ -751,11 +785,11 @@ static int mysql_check_ts(MYSQL_ROW row) {
         sprintf(session_id, "%.8s", row[1]);
 
         s = CHECKRAD2;
-        execl(CHECKRAD2, "checkrad", nas->nastype, row[3], row[4],
+        execl(CHECKRAD2, "checkrad", nas->nastype, row[4], row[5],
                 row[2], session_id, NULL);
         if (errno == ENOENT) {
                 s = CHECKRAD1;
-                execl(CHECKRAD1, "checklogin", nas->nastype, row[3], row[4],
+                execl(CHECKRAD1, "checklogin", nas->nastype, row[4], row[5],
                         row[2], session_id, NULL);
         }
         log(L_ERR, "Check-TS: exec %s: %s", s, strerror(errno));
@@ -1174,6 +1208,71 @@ int mysql_dict_init(void) {
 		return(0);
 }
 
+ /*
+  *    Read the realms table.
+  *
+  */
+int mysql_read_realms(char *realmtable) {
+
+   char		querystr[2048];
+   MYSQL_RES	*result;
+   MYSQL_ROW	row;
+   
+   realm_free(realms);
+   realms = NULL;
+   
+   if (!mysql_checksocket("Auth"))
+     return -1;
+   
+   /* Maybe realm.nas should refer exact to either nas.nasname 
+      or nas.shortname. Either way, GROUP BY makes the realmnames 
+      unique - CS */
+   sprintf(querystr, "SELECT realmname, nasname, ipaddr, authport, options FROM %s, %s WHERE nas = nasname OR nas = shortname or nas = ipaddr GROUP BY realmname", realmtable, mysql_nas_table);
+   
+   if (sqltrace)
+     DEBUG(querystr);
+   mysql_query(MyAuthSock, querystr);
+   
+   if (!(result = mysql_store_result(MyAuthSock)) && 
+       mysql_num_fields(MyAuthSock)) 
+     {
+       log(L_ERR,"MYSQL Error: Cannot get result");
+       log(L_ERR,"MYSQL error: %s",mysql_error(MyAuthSock));
+       mysql_close(MyAuthSock);
+       MyAuthSock = NULL;
+     } 
+   else 
+     {
+       REALM   *c;
+       
+       while ((row = mysql_fetch_row(result))) 
+       {
+         if ((c = malloc(sizeof(REALM))) == NULL) 
+           {
+             log(L_CONS|L_ERR, "out of memory");
+             return -1;
+           }
+        
+	strcpy(c->realm, row[0]);
+	strcpy(c->server, row[1]);
+	c->ipaddr = ipstr2long(row[2]);
+	if (strcmp(row[3],"0") == 0)
+		c->auth_port = auth_port;
+	else
+		c->auth_port = strtol(row[3], NULL, 0);
+	c->acct_port = c->auth_port + 1;
+	c->striprealm = strcmp(row[4], "nostrip") != 0;
+ 
+         c->next = realms;
+         realms = c;
+       }
+       
+       mysql_free_result(result);
+     }
+   return 0;
+ }
+
 
 /* ENDIF from top */
 #endif
+
