@@ -220,7 +220,7 @@ int main(int argc, char **argv)
 	/*
 	 *	Process the options.
 	 */
-	while((argval = getopt(argc, argv, "ASac:i:l:d:bfp:svxyz")) != EOF) {
+	while((argval = getopt(argc, argv, "ASa:ci:l:d:bfp:svxyz")) != EOF) {
 
 		switch(argval) {
 
